pub enum WithinLoop {
None,
When,
Body,
}Expand description
Whether we’re in a loop, and if so, its details
Set only for expansions of a RepeatedTemplate,
not any kind of implicit looping eg dbg_all_keywords, vpat, etc.
At some future point this may have enough information
to provide $loop_index, etc.
Right now it’s only used for dbg_all_keywords.
Variants