Context

Struct Context 

pub struct Context<'c> {
    pub top: &'c DeriveInput,
    pub template_crate: &'c Path,
    pub template_name: Option<&'c Path>,
    pub pmetas: &'c Vec<PreprocessedValueList>,
    pub variant: Option<&'c WithinVariant<'c>>,
    pub field: Option<&'c WithinField<'c>>,
    pub within_loop: WithinLoop,
    pub pvariants: &'c [PreprocessedVariant<'c>],
    pub defs: DefinitionsContext<'c>,
}
Context during expansion

References the driver, and digested information about it. Also represents where in the driver we are, including repetition context.

Fields

top: &'c DeriveInputtemplate_crate: &'c Pathtemplate_name: Option<&'c Path>pmetas: &'c Vec<PreprocessedValueList>variant: Option<&'c WithinVariant<'c>>field: Option<&'c WithinField<'c>>within_loop: WithinLooppvariants: &'c [PreprocessedVariant<'c>]defs: DefinitionsContext<'c>

Implementations

Source

impl<'c> Context<'c>

pub fn is_enum(&self) -> bool

pub fn expansion_description(&self) -> impl Display

Description of the whole expansion, suitable for dbg option, etc.

Source

impl<'c> Context<'c>

pub fn display_for_dbg(&self) -> impl Display + '_

Source

impl<'c> Context<'c>

pub fn as_general(&'c self) -> GeneralContext<'c>

Source

impl<'c> Context<'c>

pub fn decode_update_metas_used(&self, input: ParseStream<'_>) -> Result<()>

Source

impl<'c> Context<'c>

pub fn encode_metas_used(&self) -> Group

Returns [::Variant .field () ...]

Source

impl<'c> Context<'c>

pub(crate) fn check_metas_used( &self, errors: &mut ErrorAccumulator, recog: &Recognised, )

Source

impl<'c> Context<'c>

pub fn error_loc(&self) -> (Span, &'static str)

Returns an ErrorLoc for the current part of the driver

pub fn for_with_within<'w, W, F, E>(&'c self, call: F) -> Result<(), E>
where W: WithinRepeatLevel<'w>, F: FnMut(&Context<'_>, &W) -> Result<(), E>, 'c: 'w,

Obtains the relevant Within(s), and calls call for each one

If there is a current W, simply calls call. Otherwise, iterates over all of them and calls call for each one.

fn within_level<W>(&'c self, why: &dyn Spanned) -> Result<&'c W>
where W: WithinRepeatLevel<'c>,

Obtains the current Within of type W

Demands that there actually is a current container W. If we aren’t, calls it an error.

pub fn field(&self, why: &dyn Spanned) -> Result<&WithinField<'_>>

Obtains the current field (or calls it an error)

pub fn variant(&self, why: &dyn Spanned) -> Result<&WithinVariant<'_>>

Obtains the current variant (or calls it an error)

pub fn syn_variant(&self, why: &dyn Spanned) -> Result<&Variant>

Obtains the current variant as a syn::Variant

Source

impl<'c> Context<'c>

pub fn call<T>( driver: &DeriveInput, template_crate: &Path, template_name: Option<&Path>, f: impl FnOnce(Context<'_>) -> Result<T>, ) -> Result<T, Error>

Calls f with a top-level Context for a syn::DeriveInput

Context has multiple levels of references to values created here, so we can’t easily provide Context::new().