ExpandInfallible

Trait ExpandInfallible 

pub trait ExpandInfallible<O> {
    // Required method
    fn expand<'c>(&self, ctx: GeneralContext<'c>, out: &mut O);
}
Convenience trait providing fn expand(self), infallible version

Some of our expand functions always record errors within the output accumulator and therefore do not need to return them.

Required Methods

fn expand<'c>(&self, ctx: GeneralContext<'c>, out: &mut O)

Implementors