pub struct TokenAccumulator(Result<TokenStream, Error>);Expand description
Accumulates tokens, or errors
We collect all the errors, and if we get an error, don’t write
anything out.
This is because compile_error! (from into_compile_error)
only works in certain places in Rust syntax (!)
Tuple Fields
0: Result<TokenStream, Error>Implementations
Sourceimpl TokenAccumulator
impl TokenAccumulator
pub fn new() -> Self
pub fn with_tokens<R>( &mut self, f: impl FnOnce(&mut TokenStream) -> R, ) -> Option<R>
pub fn append(&mut self, t: impl ToTokens)
pub fn tokens(self) -> Result<TokenStream>
Sourcepub fn append_maybe_punct_composable(
&mut self,
val: &(impl ToTokens + ToTokensPunctComposable),
composable: bool,
)
pub fn append_maybe_punct_composable( &mut self, val: &(impl ToTokens + ToTokensPunctComposable), composable: bool, )
Appends val, via ToTokensPunctComposable or ToTokens